@@ -21,8 +21,10 @@ class FieldSet(object):
     """
     def __init__(self, U, V, fields={}):
         self.gridset = GridSet()
-        self.add_field(U)
-        self.add_field(V)
+        if U:
+            self.add_field(U)
+        if V:
+            self.add_field(V)
         UV = Field('UV', None)
         UV.fieldset = self
         self.UV = UV
@@ -32,12 +34,17 @@ class FieldSet(object):
             self.add_field(field)
 
     @classmethod
-    def from_data(cls, data, dimensions, transpose=True, mesh='spherical',
+    def from_data(cls, data, dimensions, transpose=False, mesh='spherical',
                   allow_time_extrapolation=True, time_periodic=False, **kwargs):
         """Initialise FieldSet object from raw data
 
         :param data: Dictionary mapping field names to numpy arrays.
                Note that at least a 'U' and 'V' numpy array need to be given
+               1. If data shape is [xdim, ydim], [xdim, ydim, zdim], [xdim, ydim, tdim] or [xdim, ydim, zdim, tdim],
+                  whichever is relevant for the dataset, use the flag transpose=True
+               2. If data shape is [ydim, xdim], [zdim, ydim, xdim], [tdim, ydim, xdim] or [tdim, zdim, ydim, xdim],
+                  use the flag transpose=False (default value)
+               3. If data has any other shape, you first need to reorder it
         :param dimensions: Dictionary mapping field dimensions (lon,
                lat, depth, time) to numpy arrays.
                Note that dimensions can also be a dictionary of dictionaries if
@@ -85,10 +92,16 @@ class FieldSet(object):
     def check_complete(self):
         assert(self.U), ('U field is not defined')
         assert(self.V), ('V field is not defined')
+        ugrid = self.U.grid
+        for g in self.gridset.grids:
+            g.check_zonal_periodic()
+            if g is not ugrid and g.time_origin != 0:
+                g.time = g.time + (g.time_origin - ugrid.time_origin) / np.timedelta64(1, 's')
+                g.time_origin = ugrid.time_origin
 
     @classmethod
     def from_netcdf(cls, filenames, variables, dimensions, indices={},
-                    mesh='spherical', allow_time_extrapolation=False, time_periodic=False, **kwargs):
+                    mesh='spherical', allow_time_extrapolation=False, time_periodic=False, full_load=False, **kwargs):
         """Initialises FieldSet object from NetCDF files
 
         :param filenames: Dictionary mapping variables to file(s). The
@@ -114,6 +127,10 @@ class FieldSet(object):
                (i.e. beyond the last available time snapshot)
         :param time_periodic: boolean whether to loop periodically over the time component of the FieldSet
                This flag overrides the allow_time_interpolation and sets it to False
+        :param full_load: boolean whether to fully load the data or only pre-load them. (default: False)
+               It is advised not to fully load the data, since in that case Parcels deals with
+               a better memory management during particle set execution.
+               full_load is however sometimes necessary for plotting the fields.
         """
 
         fields = {}
@@ -136,9 +153,9 @@ class FieldSet(object):
 
             fields[var] = Field.from_netcdf(var, dims, paths, inds, mesh=mesh,
                                             allow_time_extrapolation=allow_time_extrapolation,
-                                            time_periodic=time_periodic, **kwargs)
-        u = fields.pop('U')
-        v = fields.pop('V')
+                                            time_periodic=time_periodic, full_load=full_load, **kwargs)
+        u = fields.pop('U', None)
+        v = fields.pop('V', None)
         return cls(u, v, fields=fields)
 
     @classmethod
@@ -216,7 +233,7 @@ class FieldSet(object):
 
     @classmethod
     def from_parcels(cls, basename, uvar='vozocrtx', vvar='vomecrty', indices={}, extra_fields={},
-                     allow_time_extrapolation=False, time_periodic=False, **kwargs):
+                     allow_time_extrapolation=False, time_periodic=False, full_load=False, **kwargs):
         """Initialises FieldSet data from NetCDF files using the Parcels FieldSet.write() conventions.
 
         :param basename: Base name of the file(s); may contain
@@ -229,6 +246,10 @@ class FieldSet(object):
                (i.e. beyond the last available time snapshot)
         :param time_periodic: boolean whether to loop periodically over the time component of the FieldSet
                This flag overrides the allow_time_interpolation and sets it to False
+        :param full_load: boolean whether to fully load the data or only pre-load them. (default: False)
+               It is advised not to fully load the data, since in that case Parcels deals with
+               a better memory management during particle set execution.
+               full_load is however sometimes necessary for plotting the fields.
         """
 
         dimensions = {}
@@ -242,7 +263,7 @@ class FieldSet(object):
                           for v in extra_fields.keys()])
         return cls.from_netcdf(filenames, indices=indices, variables=extra_fields,
                                dimensions=dimensions, allow_time_extrapolation=allow_time_extrapolation,
-                               time_periodic=time_periodic, **kwargs)
+                               time_periodic=time_periodic, full_load=full_load, **kwargs)
 
     @property
     def fields(self):
@@ -280,7 +301,7 @@ class FieldSet(object):
 
         for grid in self.gridset.grids:
             grid.add_periodic_halo(zonal, meridional, halosize)
-        for attr, value in self.__dict__.iteritems():
+        for attr, value in iter(self.__dict__.items()):
             if isinstance(value, Field):
                 value.add_periodic_halo(zonal, meridional, halosize)
 
@@ -313,6 +334,10 @@ class FieldSet(object):
         """Replace oldest time on FieldSet with new FieldSet
         :param fieldset_new: FieldSet snapshot with which the oldest time has to be replaced"""
 
+        logger.warning_once("Fieldset.advancetime() is deprecated.\n \
+                             Parcels deals automatically with loading only 3 time steps simustaneously\
+                             such that the total allocated memory remains limited.")
+
         advance = 0
         for gnew in fieldset_new.gridset.grids:
             gnew.advanced = False
@@ -330,3 +355,49 @@ class FieldSet(object):
                 advance = advance2
                 gnew.advanced = True
             f.advancetime(fnew, advance == 1)
+
+    def computeTimeChunk(self, time, dt):
+        signdt = np.sign(dt)
+        nextTime = np.infty * signdt
+
+        for g in self.gridset.grids:
+            g.update_status = 'no_update'
+        for f in self.fields:
+            if f.name == 'UV' or not f.grid.defer_load:
+                continue
+            if f.grid.update_status == 'no_update':
+                nextTime_loc = f.grid.computeTimeChunk(f, time, signdt)
+            nextTime = min(nextTime, nextTime_loc) if signdt >= 0 else max(nextTime, nextTime_loc)
+
+        for f in self.fields:
+            if f.name == 'UV' or not f.grid.defer_load:
+                continue
+            g = f.grid
+            if g.update_status == 'first_update':  # First load of data
+                data = np.empty((g.tdim, g.zdim, g.ydim-2*g.meridional_halo, g.xdim-2*g.zonal_halo), dtype=np.float32)
+                for tindex in range(3):
+                    data = f.computeTimeChunk(data, tindex)
+                if f._scaling_factor:
+                    data *= f._scaling_factor
+                f.data = f.reshape(data)
+            elif g.update_status == 'update':
+                data = np.empty((g.tdim, g.zdim, g.ydim-2*g.meridional_halo, g.xdim-2*g.zonal_halo), dtype=np.float32)
+                if signdt >= 0:
+                    f.data[:2, :] = f.data[1:, :]
+                    tindex = 2
+                else:
+                    f.data[1:, :] = f.data[:2, :]
+                    tindex = 0
+                data = f.computeTimeChunk(data, tindex)
+                if f._scaling_factor:
+                    data *= f._scaling_factor
+                f.data[tindex, :] = f.reshape(data)[tindex, :]
+
+        if abs(nextTime) == np.infty or np.isnan(nextTime):  # Second happens when dt=0
+            return nextTime
+        else:
+            nSteps = int((nextTime - time) / dt)
+            if nSteps == 0:
+                return nextTime
+            else:
+                return time + nSteps * dt
