@@ -3,6 +3,7 @@ import numpy as np
 import netCDF4
 from datetime import timedelta as delta
 from parcels.loggers import logger
+from os import path
 
 
 __all__ = ['ParticleFile']
@@ -18,11 +19,6 @@ class ParticleFile(object):
     The current implementation is based on the NCEI template:
     http://www.nodc.noaa.gov/data/formats/netcdf/v2.0/trajectoryIncomplete.cdl
 
-    Both 'Orthogonal multidimensional array' and 'Indexed ragged array' representation
-    are implemented. The former is simpler to post-process, but the latter is required
-    when particles will be added during the .execute (i.e. the number of particles in
-    the pset increases).
-
     Developer note: We cannot use xray.Dataset here, since it does not yet allow
     incremental writes to disk: https://github.com/pydata/xarray/issues/199
 
@@ -31,43 +27,30 @@ class ParticleFile(object):
     :param outputdt: Interval which dictates the update frequency of file output
                      while ParticleFile is given as an argument of ParticleSet.execute()
                      It is either a timedelta object or a positive double.
-    :param type: Either 'array' for default matrix style, or 'indexed' for indexed ragged array
     :param write_ondelete: Boolean to write particle data only when they are deleted. Default is False
     """
 
-    def __init__(self, name, particleset, outputdt=np.infty, type='array', write_ondelete=False):
+    def __init__(self, name, particleset, outputdt=np.infty, write_ondelete=False):
 
-        self.type = type
         self.name = name
         self.write_ondelete = write_ondelete
         self.outputdt = outputdt
-        if self.write_ondelete and self.type is 'array':
-            logger.warning('ParticleFile.write_ondelete=True requires type="indexed". Setting that option')
-            self.type = 'indexed'
-        self.outputdt = outputdt
+        self.lasttraj = 0  # id of last particle written
         self.lasttime_written = None  # variable to check if time has been written already
-        self.dataset = netCDF4.Dataset("%s.nc" % name, "w", format="NETCDF4")
+        extension = path.splitext(str(name))[1]
+        fname = name if extension in ['.nc', '.nc4'] else "%s.nc" % name
+        self.dataset = netCDF4.Dataset(fname, "w", format="NETCDF4")
         self.dataset.createDimension("obs", None)
-        if self.type is 'array':
-            self.dataset.createDimension("trajectory", particleset.size)
-            coords = ("trajectory", "obs")
-        elif self.type is 'indexed':
-            coords = ("obs")
-        else:
-            raise RuntimeError("ParticleFile type must be either 'array' or 'indexed'")
+        self.dataset.createDimension("trajectory", None)
+        coords = ("trajectory", "obs")
         self.dataset.feature_type = "trajectory"
         self.dataset.Conventions = "CF-1.6/CF-1.7"
         self.dataset.ncei_template_version = "NCEI_NetCDF_Trajectory_Template_v2.0"
 
         # Create ID variable according to CF conventions
-        if self.type is 'array':
-            self.id = self.dataset.createVariable("trajectory", "i4", ("trajectory",))
-            self.id.long_name = "Unique identifier for each particle"
-            self.id.cf_role = "trajectory_id"
-            self.id[:] = np.array([p.id for p in particleset])
-        elif self.type is 'indexed':
-            self.id = self.dataset.createVariable("trajectory", "i4", ("obs",))
-            self.id.long_name = "index of trajectory this obs belongs to"
+        self.id = self.dataset.createVariable("trajectory", "i4", coords)
+        self.id.long_name = "Unique identifier for each particle"
+        self.id.cf_role = "trajectory_id"
 
         # Create time, lat, lon and z variables according to CF conventions:
         self.time = self.dataset.createVariable("time", "f8", coords, fill_value=np.nan)
@@ -102,7 +85,7 @@ class ParticleFile(object):
         self.user_vars_once = []
         """
         :user_vars: list of additional user defined particle variables to write for all particles and all times
-        :user_vars_once: list of additional user defined particle variables to write for all particles only once at initial time. Only fully functional for type='array'
+        :user_vars_once: list of additional user defined particle variables to write for all particles only once at initial time.
         """
 
         for v in particleset.ptype.variables:
@@ -119,7 +102,7 @@ class ParticleFile(object):
                 getattr(self, v.name).standard_name = v.name
                 getattr(self, v.name).units = "unknown"
 
-        self.idx = 0
+        self.idx = np.empty(shape=0)
 
     def __del__(self):
         self.dataset.close()
@@ -140,48 +123,33 @@ class ParticleFile(object):
             time = time.total_seconds()
         if self.lasttime_written != time and \
            (self.write_ondelete is False or deleted_only is True):
+            if pset.size > 0:
 
-            self.lasttime_written = time
-            if self.type is 'array':
-                # Check if largest particle ID is smaller than the last ID in ParticleFile.
-                # Otherwise, new particles have been added and netcdf will fail
-                if pset.size > 0:
-                    if max([p.id for p in pset]) > self.id[-1]:
-                        logger.error("Number of particles appears to increase. Use type='indexed' for ParticleFile")
-                        exit(-1)
+                first_write = [p for p in pset if p.fileid < 0 or len(self.idx) == 0]  # len(self.idx)==0 in case pset is written to new ParticleFile
+                for p in first_write:
+                    p.fileid = self.lasttraj
+                    self.lasttraj += 1
 
-                    # Finds the indices (inds) of the particle IDs in the ParticleFile,
-                    # because particles can have been deleted
-                    pids = [p.id for p in pset]
-                    inds = np.in1d(self.id[:], pids, assume_unique=True)
-                    inds = np.arange(len(self.id[:]))[inds]
+                self.idx = np.append(self.idx, np.zeros(len(first_write)))
 
-                    self.time[inds, self.idx] = time
-                    self.lat[inds, self.idx] = np.array([p.lat for p in pset])
-                    self.lon[inds, self.idx] = np.array([p.lon for p in pset])
-                    self.z[inds, self.idx] = np.array([p.depth for p in pset])
+                for p in pset:
+                    i = p.fileid
+                    self.id[i, self.idx[i]] = p.id
+                    self.time[i, self.idx[i]] = time
+                    self.lat[i, self.idx[i]] = p.lat
+                    self.lon[i, self.idx[i]] = p.lon
+                    self.z[i, self.idx[i]] = p.depth
                     for var in self.user_vars:
-                        getattr(self, var)[inds, self.idx] = np.array([getattr(p, var) for p in pset])
-                    if self.idx == 0:
-                        for var in self.user_vars_once:
-                            getattr(self, var)[inds] = np.array([getattr(p, var) for p in pset])
-                else:
-                    logger.warning("ParticleSet is empty on writing as array")
+                        getattr(self, var)[i, self.idx[i]] = getattr(p, var)
+                for p in first_write:
+                    for var in self.user_vars_once:
+                        getattr(self, var)[p.fileid] = getattr(p, var)
+            else:
+                logger.warning("ParticleSet is empty on writing as array")
 
+            if not deleted_only:
                 self.idx += 1
-            elif self.type is 'indexed':
-                if self.user_vars_once:
-                    logger.warning("Option to_write='once' is not fully functional in indexed mode! Particle properties of such variables are not written for newly added particles.")
-                ind = np.arange(pset.size) + self.idx
-                self.id[ind] = np.array([p.id for p in pset])
-                self.time[ind] = np.array([p.time for p in pset])
-                self.lat[ind] = np.array([p.lat for p in pset])
-                self.lon[ind] = np.array([p.lon for p in pset])
-                self.z[ind] = np.array([p.depth for p in pset])
-                for var in self.user_vars:
-                    getattr(self, var)[ind] = np.array([getattr(p, var) for p in pset])
-
-                self.idx += pset.size
+                self.lasttime_written = time
 
         if sync:
             self.sync()
