@@ -14,17 +14,6 @@ from datetime import datetime
 __all__ = ['ParticleSet']
 
 
-def nearest_index(array, value):
-    """returns index of the nearest value in array using O(log n) bisection method"""
-    y = bisect.bisect(array, value)
-    if y == len(array):
-        return y-1
-    elif(abs(array[y-1] - value) < abs(array[y] - value)):
-        return y-1
-    else:
-        return y
-
-
 class ParticleSet(object):
     """Container class for storing particle and executing kernel over them.
 
@@ -61,7 +50,7 @@ class ParticleSet(object):
         time = time.tolist() if isinstance(time, np.ndarray) else time
         time = [time] * len(lat) if not isinstance(time, list) else time
         time = [t.total_seconds() if isinstance(t, delta) else t for t in time]
-        time = [(t - fieldset.U.grid.time_origin).total_seconds() if isinstance(t, datetime) else t for t in time]
+        time = [((np.datetime64(t) - fieldset.U.grid.time_origin) / np.timedelta64(1, 's')) if isinstance(t, datetime) else t for t in time]
 
         assert len(lon) == len(time)
 
@@ -325,7 +314,7 @@ class ParticleSet(object):
             next_prelease = np.infty * np.sign(dt)
         next_output = time + outputdt * np.sign(dt)
         next_movie = time + moviedt * np.sign(dt)
-        next_input = np.infty * np.sign(dt)  # Not used yet
+        next_input = self.fieldset.computeTimeChunk(time, np.sign(dt))
 
         tol = 1e-12
         while (time < endtime and dt > 0) or (time > endtime and dt < 0) or dt == 0:
@@ -339,8 +328,6 @@ class ParticleSet(object):
                                      lat=self.repeatlat, depth=self.repeatdepth,
                                      pclass=self.repeatpclass))
                 next_prelease += self.repeatdt * np.sign(dt)
-            if abs(time-next_input) < tol:
-                continue
             if abs(time-next_output) < tol:
                 if output_file:
                     output_file.write(self, time)
@@ -348,6 +335,7 @@ class ParticleSet(object):
             if abs(time-next_movie) < tol:
                 self.show(field=movie_background_field, show_time=time)
                 next_movie += moviedt * np.sign(dt)
+            next_input = self.fieldset.computeTimeChunk(time, dt)
             if dt == 0:
                 break
 
@@ -381,12 +369,24 @@ class ParticleSet(object):
         plat = np.array([p.lat for p in self])
         show_time = self[0].time if show_time is None else show_time
         if isinstance(show_time, datetime):
-            show_time = (show_time - self.fieldset.U.grid.time_origin).total_seconds()
+            show_time = np.datetime64(show_time)
+        if isinstance(show_time, np.datetime64):
+            show_time = (show_time - self.fieldset.U.grid.time_origin) / np.timedelta64(1, 's')
         if isinstance(show_time, delta):
             show_time = show_time.total_seconds()
         if np.isnan(show_time):
             show_time = self.fieldset.U.grid.time[0]
         if domain is not None:
+            def nearest_index(array, value):
+                """returns index of the nearest value in array using O(log n) bisection method"""
+                y = bisect.bisect(array, value)
+                if y == len(array):
+                    return y - 1
+                elif (abs(array[y - 1] - value) < abs(array[y] - value)):
+                    return y - 1
+                else:
+                    return y
+
             latN = nearest_index(self.fieldset.U.lat, domain[0])
             latS = nearest_index(self.fieldset.U.lat, domain[1])
             lonE = nearest_index(self.fieldset.U.lon, domain[2])
@@ -419,6 +419,7 @@ class ParticleSet(object):
             time_origin = self.fieldset.U.grid.time_origin
         else:
             time_origin = self.fieldset.U.grid.time_origin
+            self.fieldset.computeTimeChunk(show_time, 1)
             (idx, periods) = self.fieldset.U.time_index(show_time)
             show_time -= periods*(self.fieldset.U.time[-1]-self.fieldset.U.time[0])
             U = np.array(self.fieldset.U.temporal_interpolate_fullfield(idx, show_time))
@@ -468,10 +469,11 @@ class ParticleSet(object):
         if time_origin is 0:
             timestr = ' after ' + str(delta(seconds=show_time)) + ' hours'
         else:
-            timestr = ' on ' + str(time_origin + delta(seconds=show_time))
+            date_str = str(time_origin + np.timedelta64(int(show_time), 's'))
+            timestr = ' on ' + date_str[:10] + ' ' + date_str[11:19]
 
         if particles:
-            if field:
+            if field is None:
                 plt.title('Particles' + timestr)
             elif field is 'vector':
                 plt.title('Particles and velocity field' + timestr)
@@ -491,59 +493,45 @@ class ParticleSet(object):
             logger.info('Plot saved to '+savefile+'.png')
             plt.close()
 
-    def density(self, field=None, particle_val=None, relative=False, area_scale=True):
+    def density(self, field=None, particle_val=None, relative=False, area_scale=False):
         """Method to calculate the density of particles in a ParticleSet from their locations,
-        through a 2D histogram
+        through a 2D histogram.
 
         :param field: Optional :mod:`parcels.field.Field` object to calculate the histogram
-                    on. Default is `fieldset.U`
-        :param particle_val: Optional list of values to weigh each particlewith
+                      on. Default is `fieldset.U`
+        :param particle_val: Optional numpy-array of values to weigh each particle with,
+                             or string name of particle variable to use weigh particles with.
+                             Default is None, resulting in a value of 1 for each particle
         :param relative: Boolean to control whether the density is scaled by the total
-                    number of particles
+                         weight of all particles. Default is False
         :param area_scale: Boolean to control whether the density is scaled by the area
-                    (in m^2) of each grid cell"""
-        lons = [p.lon for p in self.particles]
-        lats = [p.lat for p in self.particles]
-        # Code for finding nearest vertex for each particle is currently very inefficient
-        # once cell tracking is implemented for SciPy particles, the below use of np.min/max
-        # will be replaced (see PR #111)
-        if field is not None:
-            # Kick out particles that are not within the limits of our density field
-            half_lon = (field.grid.lon[1] - field.grid.lon[0])/2
-            half_lat = (field.grid.lat[1] - field.grid.lat[0])/2
-            dparticles = (lons > (np.min(field.grid.lon)-half_lon)) * (lons < (np.max(field.grid.lon)+half_lon)) * \
-                         (lats > (np.min(field.grid.lat)-half_lat)) * (lats < (np.max(field.grid.lat)+half_lat))
-            dparticles = np.where(dparticles)[0]
-        else:
-            field = self.fieldset.U
-            dparticles = range(len(self.particles))
-        Density = np.zeros((field.grid.lon.size, field.grid.lat.size), dtype=np.float32)
+                           (in m^2) of each grid cell. Default is False
+        """
 
-        # For each particle, find closest vertex in x and y and add 1 or val to the count
-        if particle_val is not None:
-            for p in dparticles:
-                Density[np.argmin(np.abs(lons[p] - field.grid.lon)), np.argmin(np.abs(lats[p] - field.grid.lat))] \
-                    += getattr(self.particles[p], particle_val)
+        field = field if field else self.fieldset.U
+        if isinstance(particle_val, str):
+            particle_val = [getattr(p, particle_val) for p in self.particles]
         else:
-            for p in dparticles:
-                nearest_lon = np.argmin(np.abs(lons[p] - field.grid.lon))
-                nearest_lat = np.argmin(np.abs(lats[p] - field.grid.lat))
-                Density[nearest_lon, nearest_lat] += 1
-            if relative:
-                Density /= len(dparticles)
+            particle_val = particle_val if particle_val else np.ones(len(self.particles))
+        density = np.zeros((field.grid.lat.size, field.grid.lon.size), dtype=np.float32)
+
+        for pi, p in enumerate(self.particles):
+            try:  # breaks if either p.xi, p.yi, p.zi, p.ti do not exist (in scipy) or field not in fieldset
+                if p.ti[field.igrid] < 0:  # xi, yi, zi, ti, not initialised
+                    raise('error')
+                xi = p.xi[field.igrid]
+                yi = p.yi[field.igrid]
+            except:
+                _, _, _, xi, yi, _ = field.search_indices(p.lon, p.lat, p.depth, 0, 0, search2D=True)
+            density[yi, xi] += particle_val[pi]
+
+        if relative:
+            density /= np.sum(particle_val)
 
         if area_scale:
-            area = np.zeros(np.shape(field.data[0, :, :]), dtype=np.float32)
-            U = self.fieldset.U
-            V = self.fieldset.V
-            dy = (V.grid.lon[1] - V.grid.lon[0])/V.units.to_target(1, V.grid.lon[0], V.grid.lat[0], V.grid.depth[0])
-            for y in range(len(U.grid.lat)):
-                dx = (U.grid.lon[1] - U.grid.lon[0])/U.units.to_target(1, U.grid.lon[0], U.grid.lat[y], V.grid.depth[0])
-                area[y, :] = dy * dx
-            # Scale by cell area
-            Density /= np.transpose(area)
+            density /= field.cell_areas()
 
-        return Density
+        return density
 
     def Kernel(self, pyfunc, c_include=""):
         """Wrapper method to convert a `pyfunc` into a :class:`parcels.kernel.Kernel` object
